#pragma once

#include <iostream>
#include <memory>
#include <vector>

#include "rf60Xtypes.h"

#if (defined _WIN32 && defined RF60X_LIBRARY)
#define API_EXPORT __declspec(dllexport)
#else
#define API_EXPORT
#endif

namespace SDK {
namespace SCANNERS {
namespace RF60X {

/**
 * @brief sdk_version - Return info about SDK version
 * @return SDK version
 */
API_EXPORT int sdk_version();

/**
 * @brief sdk_init - Initialize sdk library
 * Must be called once before further calls to any library functions
 * @return true if success.
 */
API_EXPORT bool sdk_init();

/**
 * @brief sdk_cleanup - Cleanup resources allocated with sdk_init() function
 */
API_EXPORT void sdk_cleanup();

class API_EXPORT rf60x {
 public:
  bool connect();

  bool disconnect();

  bool get_measure_udp(void* measure);

  bool open_port_uart();

  uart_hello_t hello_msg_uart();

  void change_config_port_uart();

  bool close_port_uart();

  bool get_measure_uart(void* measure, PROTOCOL_MEASURE_UART type);

  bool get_single_measure(uint8_t address, void* measure);

  bool send_command_modbus(modbus_rtu_command& command);

  bool receive_data_modbus(uint16_t* buffer);

  bool read_params_uart();

  bool read_params_by_custom_command_uart(uint8_t command, uint8_t size,
                                          void* value);

  bool write_params_uart();

  bool write_params_code_of_parameter_uart(
      CODE::PARAM_NAME_KEY code_of_parameter, uint16_t value);

  bool write_params_by_custom_command_uart(uint8_t command, uint8_t size,
                                           void* value);

  bool send_command(COMMAND_UART CMD);

  bool set_sensor(uint8_t value);
  bool set_analog_out(uint8_t value);

  //  bool set_AL_mode();//TODO надо проработать биты
  //  bool set_averaging_mode();
  //  bool set_analog_ouput_mode();
  //  bool set_sampling_mode();

  bool set_network_address(uint8_t value);

  bool set_baute_rate(uint8_t value);

  bool set_number_of_averaged_values(uint8_t value);

  bool set_sampling_period(uint16_t value);

  bool set_maximum_integration_time(uint16_t value);

  bool set_begin_analog_range(uint16_t value);

  bool set_end_analog_range(uint16_t value);

  bool set_time_lock_of_result(uint8_t value);

  bool set_zero_point(uint16_t value);

  bool set_CAN_interface_speed(uint8_t value);

  bool set_standard_identifier(uint16_t value);

  bool set_extend_identifier(uint32_t value);

  bool set_CAN_interface_identifier(uint8_t value);

  bool set_CAN_interface(uint8_t value);

  bool set_destination_ip_address(std::string value);

  bool set_gateway_ip_address(std::string value);

  bool set_subnet_mask(std::string value);

  bool set_source_ip_address(std::string value);

  bool set_number_of_measurements_in_the_packet(uint16_t value);

  bool set_ethernet_inteface(uint8_t value);

  bool set_autostart_of_measurement_stream(uint8_t value);

  bool set_protol_type(uint8_t value);

   std::pair<bool,uint8_t>  get_sensor();
   std::pair<bool,uint8_t> get_analog_out();

  //  bool get_AL_mode();//TODO надо проработать биты
  //  bool get_averaging_mode();
  //  bool get_analog_ouput_mode();
  //  bool get_sampling_mode();

  std::pair<bool,uint8_t> get_network_address();

  std::pair<bool,uint8_t> get_baute_rate();

  std::pair<bool,uint8_t> get_number_of_averaged_values();

  std::pair<bool,uint16_t> get_sampling_period();

  std::pair<bool,uint16_t>  get_maximum_integration_time();

  std::pair<bool,uint16_t>  get_begin_analog_range();

  std::pair<bool,uint16_t> get_end_analog_range();

  std::pair<bool,uint8_t> get_time_lock_of_result();

  std::pair<bool,uint16_t> get_zero_point();

  std::pair<bool,uint8_t> get_CAN_interface_speed();

  std::pair<bool,uint16_t> get_standard_identifier();

  std::pair<bool,uint32_t> get_extend_identifier();

  std::pair<bool,uint8_t> get_CAN_interface_identifier();

  std::pair<bool,uint8_t> get_CAN_interface();

  std::pair<bool,std::string> get_destination_ip_address();

  std::pair<bool,std::string> get_gateway_ip_address();

  std::pair<bool,std::string> get_subnet_mask();

   std::pair<bool,std::string> get_source_ip_address();

  std::pair<bool,uint16_t> get_number_of_measurements_in_the_packet();

  std::pair<bool,uint8_t> get_ethernet_inteface();

  std::pair<bool,uint8_t> get_autostart_of_measurement_stream();

  std::pair<bool,uint8_t> get_protol_type();

  param_t* get_param(std::string param_name);
  param_t* get_param(PARAM_NAME_KEY param_name);

  bool set_param(param_t* param);
  bool set_param(const char* param_name, ...);
  bool set_param(int param_id, ...);

  rf60x(const config_base_information_rf60x_t& config);
  rf60x(const rf60x&& obj) = delete;
  rf60x(const rf60x& obj) = delete;
  rf60x& operator=(const rf60x& x) = delete;
  rf60x& operator=(rf60x&&) = delete;
  ~rf60x();

 private:
  void* scanner_base = nullptr;
  uint8_t m_count_modbus;
  config_base_information_rf60x_t current_config;
};
}  // namespace RF60X
}  // namespace SCANNERS
}  // namespace SDK
